/**
 * @description skip sponsored segments in YouTube video
 * @author halcyon
 * @version 1.0.0
 * @license MIT
 * @interface {skip_sponsors: true, skip_self_promo: true, skip_poi_highlight: true} / localStorage
 */

(function () {

try {

if (window.skip_sponsors_injected) return;

// sha256
var sha256=function a(b){function c(a,b){return a>>>b|a<<32-b}for(var d,e,f=Math.pow,g=f(2,32),h="length",i="",j=[],k=8*b[h],l=a.h=a.h||[],m=a.k=a.k||[],n=m[h],o={},p=2;64>n;p++)if(!o[p]){for(d=0;313>d;d+=p)o[d]=p;l[n]=f(p,.5)*g|0,m[n++]=f(p,1/3)*g|0}for(b+="\x80";b[h]%64-56;)b+="\x00";for(d=0;d<b[h];d++){if(e=b.charCodeAt(d),e>>8)return;j[d>>2]|=e<<(3-d)%4*8}for(j[j[h]]=k/g|0,j[j[h]]=k,e=0;e<j[h];){var q=j.slice(e,e+=16),r=l;for(l=l.slice(0,8),d=0;64>d;d++){var s=q[d-15],t=q[d-2],u=l[0],v=l[4],w=l[7]+(c(v,6)^c(v,11)^c(v,25))+(v&l[5]^~v&l[6])+m[d]+(q[d]=16>d?q[d]:q[d-16]+(c(s,7)^c(s,18)^s>>>3)+q[d-7]+(c(t,17)^c(t,19)^t>>>10)|0),x=(c(u,2)^c(u,13)^c(u,22))+(u&l[1]^u&l[2]^l[1]&l[2]);l=[w+x|0].concat(l),l[4]=l[4]+w|0}for(d=0;8>d;d++)l[d]=l[d]+r[d]|0}for(d=0;8>d;d++)for(e=3;e+1;e--){var y=l[d]>>8*e&255;i+=(16>y?0:"")+y.toString(16)}return i}; /*https://geraintluff.github.io/sha256/sha256.min.js (public domain)*/


const getVideoID = (url) => {
  const match = url.match(/(?:v=|\/)([a-zA-Z0-9_-]{11})/);
  return match ? match[1] : null;
}

// default option is to skip all
const buildCategories = () => {
  const preferences = JSON.parse(localStorage.getItem("preferences")) || {
    skip_sponsors: true,
    skip_self_promo: true,
    skip_poi_highlight: true,
  };
  const categories = [];
  if (preferences.skip_sponsors) categories.push('sponsor');
  if (preferences.skip_self_promo) categories.push('selfpromo');
  if (preferences.skip_poi_highlight) categories.push('poi_highlight');
  return categories;
}


const categories = buildCategories();

const fetchSkipSegments = async (videoID) => {
  const hash = sha256(videoID).substring(0, 4);
  const url = `https://sponsor.ajay.app/api/skipSegments/${hash}?service=YouTube&categories=${encodeURIComponent(
    JSON.stringify(categories)
  )}`;

  try {
    const response = await fetch(url);
    if (!response.ok) {
      console.error(`HTTP error! Sponsorblock server down! Status: ${response.status}`);
      return [];
    }
    const data = await response.json();
    if (!Array.isArray(data)) {
      console.error("Unexpected response format");
      return [];
    }
    return data;
  } catch (e) {
    console.error('Error fetching data:', error);
    return [];
  }
}


let isProcessing = false;

const processSegments = () => {
  const video = document.querySelector('video');
  const originalSegmentSlider = document.querySelector(".ytProgressBarLineProgressBarPlayed");
  const sliderContainer = document.querySelector(".ytProgressBarLineProgressBarLine");
  if (!video || video.currentTime <= 0 || !originalSegmentSlider || !sliderContainer || document.querySelector(".xxx-sponsor-slider") || !window.location.href.includes('watch?v=') || isProcessing) {
    return;
  }
  isProcessing = true;
  const videoID = getVideoID(window.location.href);
  if (!videoID) return;
  
  const skipSegments = []
  
  fetchSkipSegments(videoID).then((segmentsList) => {
    for (const segments of segmentsList) {
      if (videoID === segments.videoID) {
        for (const segment of segments.segments) {
          // slider for segments
          let slider = originalSegmentSlider.cloneNode(true);
          if (categories.includes(segment.category)) {
            slider.style.setProperty("background-image", "none", "important");
            switch (segment.category) {
              case "sponsor":
                slider.style.setProperty("background-color", "green", "important");
                break;
              case "selfpromo":
                slider.style.setProperty("background-color", "yellow", "important");
                break
              case "poi_highlight":
                slider.style.setProperty("background-color", "blue", "important");
                break
            }
            // avoid duplicate sliders
            slider.classList.add("xxx-sponsor-slider");
            slider.style.left = `${100 * segment.segment[0] / video.duration}%`;
            slider.style.width = `${100 * (segment.segment[1] - segment.segment[0]) / video.duration}%`;
            sliderContainer.appendChild(slider)
            skipSegments.push(segment.segment);
          } else {
            console.warn("unexpected category", segment.category);
          }
        }
      }
    }

    // listen video timeupdate event and skip segments
    video.addEventListener('timeupdate', () => {
      skipSegments.forEach(segment => {
        const endTime = segment[1];
        if (video.currentTime >= segment[0] && video.currentTime < endTime) {
          video.currentTime = endTime;
        }
      });
    });
    isProcessing = false;
  });
}

setInterval(processSegments, 1000);

// make sure script is totally injected
window.skip_sponsors_injected = true;

} catch (e) {
  console.error('Error skip sponsors:', e);
}

})()
